"""
Get metada data for the courts covered by the service
"""

import pathlib
from datetime import date

from ruamel.yaml import YAML


class Court:
    def __init__(self, data):
        self.code = data.get("code")
        self.name = data.get("name")
        self.list_name = data.get("list_name") or data.get("name")
        self.link = data.get("link")
        self.ncn = data.get("ncn")
        self.canonical_param = data.get("param")
        self.param_aliases = [data.get("param")] + (data.get("extra_params") or [])
        self.start_year = data.get("start_year")
        self.end_year = data.get("end_year") or date.today().year


class CourtGroup:
    def __init__(self, name, courts):
        self.name = name
        self.courts = courts


class CourtsRepository:
    def __init__(self, data):
        self._data = data

    def get_by_param(self, param):
        for group in self._data:
            for court in group.get("courts"):
                if court.get("param") == param:
                    return Court(court)

    def get_all(self):
        return [
            Court(court) for category in self._data for court in category.get("courts")
        ]

    def get_selectable(self):
        courts = []
        for category in self._data:
            for court in category.get("courts"):
                if court.get("selectable"):
                    courts.append(Court(court))
        return courts

    def get_listable_groups(self):
        groups = []
        for category in self._data:
            courts = [
                Court(court)
                for court in category.get("courts")
                if court.get("listable")
            ]
            if len(courts) > 0:
                groups.append(CourtGroup(category.get("display_name"), courts))
        return groups

    def get_listable_courts(self):
        courts = []
        for group in self._data:
            if not group.get("is_tribunal"):
                for court in group.get("courts", []):
                    if court.get("listable"):
                        courts.append(Court(court))
        return courts

    def get_listable_tribunals(self):
        courts = []
        for group in self._data:
            if group.get("is_tribunal"):
                for court in group.get("courts", []):
                    if court.get("listable"):
                        courts.append(Court(court))
        return courts


yaml = YAML()
datafile = pathlib.Path(__file__).parent / "data/court_names.yaml"
with open(datafile) as f:
    court_data = yaml.load(f)

courts = CourtsRepository(court_data)



import pathlib
import unittest
from datetime import date

from ruamel.yaml import YAML

from .courts import Court, CourtsRepository, courts


class TestCourtsRepository(unittest.TestCase):
    def test_loads_selectable_courts(self):
        data = [
            {
                "name": "court_group",
                "courts": [
                    {
                        "name": "court1",
                        "selectable": True,
                    },
                    {"name": "court2", "selectable": False},
                ],
            }
        ]
        repo = CourtsRepository(data)
        selectable = repo.get_selectable()
        self.assertIn("court1", [c.name for c in selectable])
        self.assertNotIn("court2", [c.name for c in selectable])

    def test_loads_listable_courts(self):
        data = [
            {
                "name": "court_group1",
                "display_name": "court group 1",
                "courts": [
                    {
                        "name": "court1",
                        "listable": True,
                    },
                    {"name": "court2", "listable": False},
                ],
            },
            {
                "name": "court_group2",
                "display_name": "court group 2",
                "courts": [{"name": "court3", "listable": False}],
            },
        ]
        repo = CourtsRepository(data)
        groups = repo.get_listable_groups()
        self.assertIn("court group 1", [g.name for g in groups])
        self.assertNotIn("court group 2", [g.name for g in groups])
        self.assertIn("court1", [c.name for g in groups for c in g.courts])
        self.assertNotIn("court2", [c.name for g in groups for c in g.courts])
        self.assertNotIn("court3", [c.name for g in groups for c in g.courts])

    def test_loads_court_by_param(self):
        data = [
            {
                "name": "court_group1",
                "courts": [{"param": "court1", "name": "Court 1"}],
            },
            {
                "name": "court_group2",
                "courts": [{"param": "court2", "name": "Court 2"}],
            },
        ]
        repo = CourtsRepository(data)
        self.assertEqual("Court 2", repo.get_by_param("court2").name)

    def test_returns_listable_courts(self):
        data = [
            {
                "name": "court_group1",
                "is_tribunal": False,
                "courts": [
                    {"param": "court1", "listable": True, "name": "Court 1"},
                    {"param": "court2", "listable": False, "name": "Court 2"},
                ],
            },
            {
                "name": "court_group2",
                "is_tribunal": True,
                "courts": [{"param": "court3", "listable": True, "name": "Court 3"}],
            },
        ]
        repo = CourtsRepository(data)
        self.assertIn("court1", [c.canonical_param for c in repo.get_listable_courts()])
        self.assertNotIn(
            "court2", [c.canonical_param for c in repo.get_listable_courts()]
        )
        self.assertNotIn(
            "court3", [c.canonical_param for c in repo.get_listable_courts()]
        )

    def test_returns_listable_tribunals(self):
        data = [
            {
                "name": "court_group1",
                "is_tribunal": False,
                "courts": [
                    {"param": "court1", "listable": True, "name": "Court 1"},
                ],
            },
            {
                "name": "court_group2",
                "is_tribunal": True,
                "courts": [
                    {"param": "court2", "listable": False, "name": "Court 2"},
                    {"param": "court3", "listable": True, "name": "Court 3"},
                ],
            },
        ]
        repo = CourtsRepository(data)
        self.assertNotIn(
            "court1", [c.canonical_param for c in repo.get_listable_tribunals()]
        )
        self.assertNotIn(
            "court2", [c.canonical_param for c in repo.get_listable_tribunals()]
        )
        self.assertIn(
            "court3", [c.canonical_param for c in repo.get_listable_tribunals()]
        )


class TestCourt(unittest.TestCase):
    def test_list_name_explicit(self):
        court = Court({"list_name": "court_name"})
        self.assertEqual("court_name", court.list_name)

    def test_list_name_default(self):
        court = Court({"name": "court_name"})
        self.assertEqual("court_name", court.list_name)

    def test_param_aliases(self):
        court = Court({"param": "param_1", "extra_params": ["param_2"]})
        self.assertEqual(["param_1", "param_2"], court.param_aliases)

    def test_end_year_explicit(self):
        court = Court({"end_year": 1983})
        self.assertEqual(1983, court.end_year)

    def test_end_year_default(self):
        court = Court({})
        self.assertEqual(date.today().year, court.end_year)


class TestCourts(unittest.TestCase):
    def test_loads_court_yaml(self):
        yaml = YAML()
        datafile = pathlib.Path(__file__).parent / "data/court_names.yaml"
        with open(datafile) as f:
            court_data = yaml.load(f)
        courts_from_yaml = []
        for group in court_data:
            for court in group.get("courts"):
                courts_from_yaml.append(court)
        for (court, data) in zip(courts.get_all(), courts_from_yaml):
            self.assertEqual(court.name, data["name"])



"""
Convert neutral Citations to URL
"""

import pathlib
import re

from ruamel.yaml import YAML

yaml = YAML()
datafile = pathlib.Path(__file__).parent / "data/neutral_citation_regex.yaml"
with open(datafile) as f:
    citation_data = yaml.load(f)


def neutral_url(citation):
    """Given a neutral citation such as `[2020] EAT 17`,
    return a public-API URL like `/eat/2020/17`, or None
    if no match is found.
    """
    for regex, groups in citation_data:
        if match := re.match(regex, citation):
            url_components = "/".join([match.groups()[x - 1] for x in groups])
            return f"/{url_components}".lower()
    return None



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['ds_caselaw_utils']

package_data = \
{'': ['*'], 'ds_caselaw_utils': ['data/*']}

install_requires = \
['ruamel.yaml>=0.17.21,<0.18.0']

setup_kwargs = {
    'name': 'ds-caselaw-utils',
    'version': '0.4.1',
    'description': 'Utilities for the National Archives Caselaw project',
    'long_description': '# Caselaw utility functions\n\npypi name: [ds-caselaw-utils](https://pypi.org/project/ds-caselaw-utils)\npython import name: `ds_caselaw_utils`\n\nThis repo contains functions of general use throughout the National Archives Caselaw project\nso that we can have a single point of truth potentially across many repositories.\n\n## Examples\n\n```\nfrom ds_caselaw_utils import neutral_url\nneutral_url("[2022] EAT 1")  # \'/eat/2022/4\'\n\nfrom ds_caselaw_utils import courts\n\ncourts.get_all() # return a list of all courts\n\ncourts.get_by_param("ewhc/ch") # get a court by its parameter value\n\ncourts.get_selectable() # returns a list of all courts that are whitelisted to\n                        # appear as searchable options\n\ncourts.get_listable_groups() # returns a grouped list of courts that are whitelisted to\n                             # be listed publicly\n\ncourts.get_listable_courts() # returns a list of all *courts* (ie not tribunals)\n                             # which are whitelisted to be listed publicly\n\ncourts.get_listable_tribunals() # return a list of all *tribunals*  which are\n                                # whitelisted to be listed publicly\n\n\n```\n\nThe list of courts is defined in `src/ds_caselaw_utils/data/court_names.yml`. The format is as follows:\n\n```\n- name: high_court # Internal name of a group of courts to be displayed together\n  display_name: "High Court" # An optional public facing name for this group.\n  is_tribunal: false # Whether this group contains courts or tribunals\n  courts: # List of courts to be displayed under this group\n    -\n        # An internal code for this court:\n        code: EWHC-SeniorCourtsCosts\n         # The public facing name of the court:\n        name: Senior Courts Costs Office\n        # An alternative wording for use in listings (optional, defaults to `name`)\n        list_name: High Court (Senior Court Costs Office)\n        # A URL to link to for more information on this court:\n        link: https://www.gov.uk/courts-tribunals/senior-courts-costs-office\n        # A regex matching neutral citations for this court\'s judgments:\n        ncn: \\[(\\d{4})\\] (EWHC) (\\d+) \\((SCCO)\\)\n        # The canonical parameter value used in searches for this court:\n        param: \'ewhc/scco\'\n        # Any additional parameter aliases which display judgments from this court:\n        extra_params: [\'ewhc/costs\']\n        # The year of the first judgment we have on file for this court:\n        start_year: 2003\n        # The year of the last judgment we have on file for this court\n        # (optional, defaults to current year):\n        end_year: ~\n        # Whether to expose this court publicly as selectable in search filters:\n        selectable: true\n        # Whether to expose this court publicly in listings:\n        listable: true\n```\n\n## Testing\n\n```bash\n$ poetry shell\n$ cd src/ds_caselaw_utils\n$ python -m unittest\n```\n\n## Building\n\n```bash\n$ rm -rf dist\n$ poetry build\n$ python3 -m twine upload --repository testpypi dist/* --verbose\n```\n\n## Releasing\n\nWhen making a new release, update the [changelog](CHANGELOG.md) in the release\npull request.\n\nThe package will **only** be released to PyPI if the branch is tagged. A merge\nto main alone will **not** trigger a release to PyPI.\n\nTo create a release:\n\n0. Update the version number in `pyproject.toml`\n1. Create a branch `release/v{major}.{minor}.{patch}`\n2. Update changelog for the release\n3. Commit and push\n4. Open a PR from that branch to main\n5. Get approval on the PR\n6. Tag the HEAD of the PR `v{major}.{minor}.{patch}` and push the tag\n7. Merge the PR to main and push\n',
    'author': 'David McKee',
    'author_email': 'dragon@dxw.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/nationalarchives/ds-caselaw-utils',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import unittest

from .neutral import neutral_url


class TestNeutralURL(unittest.TestCase):
    def test_good_neutral_urls(self):
        self.assertEqual(neutral_url("[2022] UKSC 1"), "/uksc/2022/1")
        self.assertEqual(neutral_url("[1604] EWCA Crim 555"), "/ewca/crim/1604/555")
        self.assertEqual(neutral_url("[2022] EWHC 1 (Comm)"), "/ewhc/comm/2022/1")
        self.assertEqual(neutral_url("[1999] EWCOP 7"), "/ewcop/1999/7")
        self.assertEqual(neutral_url("[2022] UKUT 1 (IAC)"), "/ukut/iac/2022/1")
        self.assertEqual(neutral_url("[2022] EAT 1"), "/eat/2022/1")
        self.assertEqual(neutral_url("[2022] UKFTT 1 (TC)"), "/ukftt/tc/2022/1")
        self.assertEqual(neutral_url("[2022] UKFTT 1 (GRC)"), "/ukftt/grc/2022/1")
        self.assertEqual(neutral_url("[2022] EWHC 1 (KB)"), "/ewhc/kb/2022/1")

    def test_bad_neutral_urls(self):
        self.assertEqual(neutral_url(""), None)
        self.assertEqual(neutral_url("1604] EWCA Crim 555"), None)
        self.assertEqual(neutral_url("[2022 EWHC 1 Comm"), None)
        self.assertEqual(neutral_url("[1999] EWCOP"), None)
        self.assertEqual(
            neutral_url("[2022] UKUT B1 IAC"), None
        )  # Could be a Bailii reference, might want to drop B in future.
        self.assertEqual(neutral_url("[2022] EAT A"), None)
        self.assertEqual(neutral_url("[2022] NOTACOURT 1 TC"), None)
